@@ -49,6 +49,7 @@ module Graphics.Vty ( Vty(..)
                     , module Graphics.Vty.Output
                     , module Graphics.Vty.Picture
                     , DisplayRegion
+                    , Mode(..)
                     ) 
     where
 
@@ -57,6 +58,7 @@ import Graphics.Vty.Prelude
 import Graphics.Vty.Config
 import Graphics.Vty.Input
 import Graphics.Vty.Output
+import Graphics.Vty.Output.Interface
 import Graphics.Vty.Picture
 
 import Control.Concurrent.STM
@@ -127,48 +129,36 @@ intMkVty input out = do
     lastUpdateRef <- newIORef Nothing
 
     let innerUpdate inPic = do
-            b@(w,h) <- displayBounds out
-            let cursor  = picCursor inPic
-                inPic' = case cursor of
-                  Cursor x y ->
-                    let
-                       x'      = case x of
-                                   _ | x < 0     -> 0
-                                     | x >= w    -> w - 1
-                                     | otherwise -> x
-                       y'      = case y of
-                                   _ | y < 0     -> 0
-                                     | y >= h    -> h - 1
-                                     | otherwise -> y
-                     in inPic { picCursor = Cursor x' y' }
-                  _ -> inPic
+            b <- displayBounds out
             mlastUpdate <- readIORef lastUpdateRef
             updateData <- case mlastUpdate of
                 Nothing -> do
                     dc <- displayContext out b
-                    outputPicture dc inPic'
+                    outputPicture dc inPic
                     return (b, dc)
                 Just (lastBounds, lastContext) -> do
                     if b /= lastBounds
                         then do
                             dc <- displayContext out b
-                            outputPicture dc inPic'
+                            outputPicture dc inPic
                             return (b, dc)
                         else do
-                            outputPicture lastContext inPic'
+                            outputPicture lastContext inPic
                             return (b, lastContext)
             writeIORef lastUpdateRef $ Just updateData
-            writeIORef lastPicRef $ Just inPic'
+            writeIORef lastPicRef $ Just inPic
 
-    let innerRefresh 
-            =   writeIORef lastUpdateRef Nothing
-            >>  readIORef lastPicRef 
-            >>= maybe ( return () ) ( \pic -> innerUpdate pic ) 
+    let innerRefresh = do
+            writeIORef lastUpdateRef Nothing
+            bounds <- displayBounds out
+            dc <- displayContext out bounds
+            writeIORef (assumedStateRef $ contextDevice dc) initialAssumedState
+            mPic <- readIORef lastPicRef
+            maybe (return ()) innerUpdate mPic
 
     let gkey = do k <- atomically $ readTChan $ _eventChannel input
                   case k of 
-                    (EvResize _ _)  -> innerRefresh
-                                       >> displayBounds out
+                    (EvResize _ _)  -> displayBounds out
                                        >>= return . (\(w,h)-> EvResize w h)
                     _               -> return k
 
