@@ -1,6 +1,12 @@
+{-# LANGUAGE CPP #-}
 {-# LANGUAGE StandaloneDeriving #-}
 {-# LANGUAGE GADTs #-}
 {-# LANGUAGE RankNTypes #-}
+
+#ifndef MIN_VERSION_base
+#defined MIN_VERSION_base(x,y,z) 1
+#endif
+
 -- | Display attributes
 --
 -- Typically the values 'defAttr' or 'currentAttr' are modified to form attributes:
@@ -33,12 +39,15 @@ module Graphics.Vty.Attributes ( module Graphics.Vty.Attributes
 
 import Data.Bits
 import Data.Default
-import Data.Monoid
 import Data.Word
 
 import Graphics.Vty.Attributes.Color
 import Graphics.Vty.Attributes.Color240
 
+#if !(MIN_VERSION_base(4,8,0))
+import Data.Monoid
+#endif
+
 -- | A display attribute defines the Color and Style of all the characters rendered after the
 -- attribute is applied.
 --
@@ -49,7 +58,7 @@ data Attr = Attr
     { attrStyle :: !(MaybeDefault Style)
     , attrForeColor :: !(MaybeDefault Color)
     , attrBackColor :: !(MaybeDefault Color)
-    } deriving ( Eq, Show )
+    } deriving ( Eq, Show, Read )
 
 -- This could be encoded into a single 32 bit word. The 32 bit word is first divided
 -- into 4 groups of 8 bits where: The first group codes what action should be taken with regards to
@@ -103,10 +112,11 @@ data FixedAttr = FixedAttr
 data MaybeDefault v where
     Default :: MaybeDefault v
     KeepCurrent :: MaybeDefault v
-    SetTo :: forall v . ( Eq v, Show v ) => !v -> MaybeDefault v
+    SetTo :: forall v . ( Eq v, Show v, Read v ) => !v -> MaybeDefault v
 
 deriving instance Eq v => Eq (MaybeDefault v)
 deriving instance Eq v => Show (MaybeDefault v)
+deriving instance (Eq v, Show v, Read v) => Read (MaybeDefault v)
 
 instance Eq v => Monoid ( MaybeDefault v ) where
     mempty = KeepCurrent
