@@ -1,7 +1,8 @@
 {-# LANGUAGE CPP #-}
-{-# LANGUAGE FlexibleContexts #-}
-{-# LANGUAGE NoMonomorphismRestriction #-}
-{-# LANGUAGE KindSignatures #-}
+{-# LANGUAGE DeriveDataTypeable #-}
+{-# LANGUAGE FlexibleContexts, FlexibleInstances #-}
+{-# LANGUAGE TypeOperators #-}
+{-# LANGUAGE ScopedTypeVariables #-}
 -- | A 'Config' can be provided to mkVty to customize the applications use of vty. A config file can
 -- be used to customize vty for a user's system.
 --
@@ -64,7 +65,16 @@
 -- Set VTY_DEBUG_LOG. Run vty. Check debug log for incorrect mappings. Add corrected mappings to
 -- .vty/config
 --
-module Graphics.Vty.Config where
+module Graphics.Vty.Config
+  ( InputMap
+  , Config(..)
+  , VtyConfigurationError(..)
+  , userConfig
+  , overrideEnvConfig
+  , standardIOConfig
+  , runParseConfig
+  , parseConfigFile
+  ) where
 
 #if __GLASGOW_HASKELL__ > 704
 import Prelude
@@ -74,17 +84,18 @@ import Prelude hiding (catch)
 
 import Control.Applicative hiding (many)
 
-import Control.Exception (catch, IOException)
-import Control.Monad (void)
-import Control.Monad.Trans.Class
-import Control.Monad.Trans.Writer
+import Control.Exception (catch, IOException, Exception(..), throwIO)
+import Control.Monad (liftM, guard, void)
 
 import qualified Data.ByteString as BS
 import Data.Default
 import Data.Monoid
+import Data.Typeable (Typeable)
 
 import Graphics.Vty.Input.Events
 
+import GHC.Generics
+
 import System.Directory (getAppUserDataDirectory)
 import System.Posix.Env (getEnv)
 import System.Posix.IO (stdInput, stdOutput)
@@ -94,6 +105,16 @@ import Text.Parsec hiding ((<|>))
 import Text.Parsec.Token ( GenLanguageDef(..) )
 import qualified Text.Parsec.Token as P
 
+-- | Type of errors that can be thrown when configuring VTY
+data VtyConfigurationError
+  = VtyMissingTermEnvVar -- ^ TERM environment variable not set
+  deriving (Show, Eq, Typeable)
+
+instance Exception VtyConfigurationError where
+#if MIN_VERSION_base(4,8,0)
+  displayException VtyMissingTermEnvVar = "TERM environment variable not set"
+#endif
+
 -- | Mappings from input bytes to event in the order specified. Later entries take precedence over
 -- earlier in the case multiple entries have the same byte string.
 type InputMap = [(Maybe String, String, Event)]
@@ -104,6 +125,10 @@ data Config = Config
       vmin  :: Maybe Int
     -- | The default is 100 milliseconds, 0.1 seconds.
     , vtime :: Maybe Int
+    -- | The default is False.
+    , mouseMode :: Maybe Bool
+    -- | The default is False.
+    , bracketedPasteMode :: Maybe Bool
     -- | Debug information is appended to this file if not Nothing.
     , debugLog           :: Maybe FilePath
     -- | The (input byte, output event) pairs extend the internal input table of VTY and the table
@@ -127,6 +152,8 @@ instance Monoid Config where
     mempty = Config
         { vmin         = Nothing
         , vtime        = Nothing
+        , mouseMode    = Nothing
+        , bracketedPasteMode = Nothing
         , debugLog     = mempty
         , inputMap     = mempty
         , inputFd     = Nothing
@@ -137,6 +164,8 @@ instance Monoid Config where
         -- latter config takes priority for everything but inputMap
         { vmin          = vmin c1     <|> vmin c0
         , vtime         = vtime c1    <|> vtime c0
+        , mouseMode     = mouseMode c1
+        , bracketedPasteMode = bracketedPasteMode c1
         , debugLog      = debugLog c1 <|> debugLog c0
         , inputMap      = inputMap c0 <>  inputMap c1
         , inputFd      = inputFd c1 <|> inputFd c0
@@ -144,8 +173,6 @@ instance Monoid Config where
         , termName      = termName c1 <|> termName c0
         }
 
-type ConfigParser s a = ParsecT s () (Writer Config) a
-
 -- | Config from @'getAppUserDataDirectory'/config@ and @$VTY_CONFIG_FILE@
 userConfig :: IO Config
 userConfig = do
@@ -159,126 +186,124 @@ overrideEnvConfig = do
     d <- getEnv "VTY_DEBUG_LOG"
     return $ def { debugLog = d }
 
+-- | Configures VTY using defaults suitable for terminals. This action
+-- can raise 'VtyConfigurationError'.
 standardIOConfig :: IO Config
 standardIOConfig = do
-    Just t <- getEnv "TERM"
-    return $ def { vmin = Just 1
-                 , vtime = Just 100
-                 , inputFd = Just stdInput
-                 , outputFd = Just stdOutput
-                 , termName = Just t
-                 }
+    mb <- getEnv "TERM"
+    case mb of
+      Nothing -> throwIO VtyMissingTermEnvVar
+      Just t ->
+        return def
+          { vmin               = Just 1
+          , mouseMode          = Just False
+          , bracketedPasteMode = Just False
+          , vtime              = Just 100
+          , inputFd            = Just stdInput
+          , outputFd           = Just stdOutput
+          , termName           = Just t
+          }
 
 parseConfigFile :: FilePath -> IO Config
 parseConfigFile path = do
     catch (runParseConfig path <$> BS.readFile path)
           (\(_ :: IOException) -> return def)
 
-runParseConfig :: Stream s (Writer Config) Char => String -> s -> Config
-runParseConfig name = execWriter . runParserT parseConfig () name
+runParseConfig :: String -> BS.ByteString -> Config
+runParseConfig name cfgTxt =
+  case runParser parseConfig () name cfgTxt of
+    Right cfg -> cfg
+    Left{}    -> def
+
+------------------------------------------------------------------------
+
+type Parser = Parsec BS.ByteString ()
 
 -- I tried to use the haskellStyle here but that was specialized (without requirement?) to the
 -- String stream type.
-configLanguage :: Stream s m Char => P.GenLanguageDef s u m
+configLanguage :: Monad m => P.GenLanguageDef BS.ByteString () m
 configLanguage = LanguageDef
-    { commentStart = "{-"
-    , commentEnd = "-}"
-    , commentLine = "--"
-    , nestedComments = True
-    , identStart = letter <|> char '_'
-    , identLetter = alphaNum <|> oneOf "_'"
-    , opStart = opLetter configLanguage
-    , opLetter = oneOf ":!#$%&*+./<=>?@\\^|-~"
+    { commentStart    = "{-"
+    , commentEnd      = "-}"
+    , commentLine     = "--"
+    , nestedComments  = True
+    , identStart      = letter <|> char '_'
+    , identLetter     = alphaNum <|> oneOf "_'"
+    , opStart         = opLetter configLanguage
+    , opLetter        = oneOf ":!#$%&*+./<=>?@\\^|-~"
     , reservedOpNames = []
-    , reservedNames = []
-    , caseSensitive = True
+    , reservedNames   = []
+    , caseSensitive   = True
     }
 
-configLexer :: Stream s m Char => P.GenTokenParser s u m
+configLexer :: Monad m => P.GenTokenParser BS.ByteString () m
 configLexer = P.makeTokenParser configLanguage
 
-mapDecl :: forall s u (m :: * -> *).
-           (Monad m, Stream s (WriterT Config m) Char) =>
-           ParsecT s u (WriterT Config m) ()
+mapDecl :: Parser Config
 mapDecl = do
-    void $ string "map"
-    P.whiteSpace configLexer
+    "map" <- P.identifier configLexer
     termIdent <- (char '_' >> P.whiteSpace configLexer >> return Nothing)
              <|> (Just <$> P.stringLiteral configLexer)
-    bytes <- P.stringLiteral configLexer
-    key <- parseKey
-    modifiers <- parseModifiers
-    lift $ tell $ def { inputMap = [(termIdent, bytes, EvKey key modifiers)] }
+    bytes     <- P.stringLiteral configLexer
+    key       <- parseValue
+    modifiers <- parseValue
+    return def { inputMap = [(termIdent, bytes, EvKey key modifiers)] }
 
--- TODO: Generated by a vim macro. There is a better way here. Derive parser? Use Read
--- instance? Generics?
-parseKey :: forall s u (m :: * -> *).
-            Stream s m Char =>
-            ParsecT s u m Key
-parseKey = do
-    key <- P.identifier configLexer
-    case key of
-     "KChar" -> KChar <$> P.charLiteral configLexer
-     "KFun" -> KFun . fromInteger <$> P.natural configLexer
-     "KEsc" -> return KEsc
-     "KBS" -> return KBS
-     "KEnter" -> return KEnter
-     "KLeft" -> return KLeft
-     "KRight" -> return KRight
-     "KUp" -> return KUp
-     "KDown" -> return KDown
-     "KUpLeft" -> return KUpLeft
-     "KUpRight" -> return KUpRight
-     "KDownLeft" -> return KDownLeft
-     "KDownRight" -> return KDownRight
-     "KCenter" -> return KCenter
-     "KBackTab" -> return KBackTab
-     "KPrtScr" -> return KPrtScr
-     "KPause" -> return KPause
-     "KIns" -> return KIns
-     "KHome" -> return KHome
-     "KPageUp" -> return KPageUp
-     "KDel" -> return KDel
-     "KEnd" -> return KEnd
-     "KPageDown" -> return KPageDown
-     "KBegin" -> return KBegin
-     "KMenu" -> return KMenu
-     _ -> fail $ key ++ " is not a valid key identifier"
-
-parseModifiers :: forall s u (m :: * -> *).
-                  Stream s m Char =>
-                  ParsecT s u m [Modifier]
-parseModifiers = P.brackets configLexer (parseModifier `sepBy` P.symbol configLexer ",")
-
-parseModifier :: forall s u (m :: * -> *).
-                 Stream s m Char =>
-                 ParsecT s u m Modifier
-parseModifier = do
-    m <- P.identifier configLexer
-    case m of
-        "KMenu" -> return MShift
-        "MCtrl" -> return MCtrl
-        "MMeta" -> return MMeta
-        "MAlt" -> return MAlt
-        _ -> fail $ m ++ " is not a valid modifier identifier"
-
-debugLogDecl :: forall s u (m :: * -> *).
-                (Monad m, Stream s (WriterT Config m) Char) =>
-                ParsecT s u (WriterT Config m) ()
+debugLogDecl :: Parser Config
 debugLogDecl = do
-    void $ string "debugLog"
-    P.whiteSpace configLexer
-    path <- P.stringLiteral configLexer
-    lift $ tell $ def { debugLog = Just path }
+    "debugLog" <- P.identifier configLexer
+    path       <- P.stringLiteral configLexer
+    return def { debugLog = Just path }
 
-ignoreLine :: forall s u (m :: * -> *).
-              Stream s m Char => ParsecT s u m ()
+ignoreLine :: Parser ()
 ignoreLine = void $ manyTill anyChar newline
 
-parseConfig :: forall s u (m :: * -> *).
-               (Monad m, Stream s (WriterT Config m) Char) =>
-               ParsecT s u (WriterT Config m) ()
-parseConfig = void $ many $ do
+parseConfig :: Parser Config
+parseConfig = liftM mconcat $ many $ do
     P.whiteSpace configLexer
-    let directives = [mapDecl, debugLogDecl]
-    try (choice directives) <|> ignoreLine
+    let directives = [try mapDecl, try debugLogDecl]
+    choice directives <|> (ignoreLine >> return def)
+
+class    Parse a        where parseValue :: Parser a
+instance Parse Char     where parseValue = P.charLiteral configLexer
+instance Parse Int      where parseValue = fromInteger <$> P.natural configLexer
+instance Parse Key      where parseValue = genericParse
+instance Parse Modifier where parseValue = genericParse
+instance Parse a => Parse [a] where
+  parseValue = P.brackets configLexer
+                 (parseValue `sepBy` P.symbol configLexer ",")
+
+------------------------------------------------------------------------
+-- Derived parser for ADTs via generics
+------------------------------------------------------------------------
+
+genericParse :: (Generic a, GParse (Rep a)) => Parser a
+genericParse = to <$> gparse
+
+class    GParse f                      where gparse :: Parser (f a)
+instance GParse f => GParse (M1 S i f) where gparse = M1 <$> gparse
+instance GParse U1                     where gparse = return U1
+instance Parse a => GParse (K1 i a)    where gparse = K1 <$> parseValue
+
+instance (GParse f, GParse g) => GParse (f :*: g) where
+  gparse = (:*:) <$> gparse <*> gparse
+
+instance GParseAlts f => GParse (M1 D i f) where
+  gparse =
+    do con <- P.identifier configLexer
+       M1 <$> gparseAlts con
+
+------------------------------------------------------------------------
+
+class GParseAlts f where
+  gparseAlts :: String -> Parser (f a)
+
+instance (Constructor i, GParse f) => GParseAlts (M1 C i f) where
+  gparseAlts con =
+    do guard (con == conName (M1 Nothing :: C1 i Maybe a))
+       M1 <$> gparse
+
+instance (GParseAlts f, GParseAlts g) => GParseAlts (f :+: g) where
+  gparseAlts con = L1 <$> gparseAlts con <|> R1 <$> gparseAlts con
+
+instance GParseAlts V1 where gparseAlts _ = fail "GParse: V1"
