@@ -12,10 +12,13 @@ module Graphics.Vty.Input.Classify
     ) where
 
 import Graphics.Vty.Input.Events
+import Graphics.Vty.Input.Mouse
+import Graphics.Vty.Input.Paste
+import Graphics.Vty.Input.Classify.Types
 
 import Codec.Binary.UTF8.Generic (decode)
 
-import Data.List(inits)
+import Data.List (inits)
 import qualified Data.Map as M( fromList, lookup )
 import Data.Maybe ( mapMaybe )
 import qualified Data.Set as S( fromList, member )
@@ -23,12 +26,6 @@ import qualified Data.Set as S( fromList, member )
 import Data.Char
 import Data.Word
 
-data KClass
-    = Valid Event [Char]
-    | Invalid
-    | Prefix
-    deriving(Show, Eq)
-
 compile :: ClassifyMap -> [Char] -> KClass
 compile table = cl' where
     -- take all prefixes and create a set of these
@@ -60,6 +57,11 @@ classify :: ClassifyMap -> [Char] -> KClass
 classify table =
     let standardClassifier = compile table
     in \s -> case s of
+        _ | bracketedPasteStarted s ->
+            if bracketedPasteFinished s
+            then parseBracketedPaste s
+            else Prefix
+        _ | isMouseEvent s   -> classifyMouseEvent s
         c:cs | ord c >= 0xC2 -> classifyUtf8 c cs
         _                    -> standardClassifier s
 
