@@ -54,17 +54,26 @@ import Data.Monoid ((<>))
 --
 -- Selection of a terminal is done as follows:
 --
---      * If TERM == xterm use XTermColor.
---      * for any other TERM value TerminfoBased is used.
+--      * If TERM contains "xterm" or "screen", use XTermColor.
+--      * otherwise use the TerminfoBased driver.
 --
 -- \todo add an implementation for windows that does not depend on terminfo. Should be installable
 -- with only what is provided in the haskell platform. Use ansi-terminal
 outputForConfig :: Config -> IO Output
 outputForConfig Config{ outputFd = Just fd, termName = Just termName, .. } = do
-    t <- if "xterm" `isPrefixOf` termName
+    t <- if "xterm" `isPrefixOf` termName || "screen" `isPrefixOf` termName
         then XTermColor.reserveTerminal termName fd
         -- Not an xterm-like terminal. try for generic terminfo.
         else TerminfoBased.reserveTerminal termName fd
+
+    case mouseMode of
+        Just s -> setMode t Mouse s
+        Nothing -> return ()
+
+    case bracketedPasteMode of
+        Just s -> setMode t BracketedPaste s
+        Nothing -> return ()
+
     return t
 outputForConfig config = (<> config) <$> standardIOConfig >>= outputForConfig
 
