@@ -37,6 +37,11 @@ import qualified Data.Vector as Vector
 import Data.Monoid (mempty, mappend)
 #endif
 
+-- | Modal terminal features that can be enabled and disabled.
+data Mode = Mouse
+          | BracketedPaste
+          deriving (Eq, Read, Show)
+
 data Output = Output
     { -- | Text identifier for the output device. Used for debugging. 
       terminalID :: String
@@ -67,11 +72,21 @@ data Output = Output
     , contextColorCount :: Int
     -- | if the cursor can be shown / hidden
     , supportsCursorVisibility :: Bool
+    -- | Indicates support for terminal modes for this output device
+    , supportsMode :: Mode -> Bool
+    -- | Enables or disables a mode (does nothing if the mode is unsupported)
+    , setMode :: forall m. MonadIO m => Mode -> Bool -> m ()
+    -- | Returns whether a mode is enabled
+    , getModeStatus :: forall m. MonadIO m => Mode -> m Bool
     , assumedStateRef :: IORef AssumedState
     -- | Acquire display access to the given region of the display.
     -- Currently all regions have the upper left corner of (0,0) and the lower right corner at 
     -- (max displayWidth providedWidth, max displayHeight providedHeight)
     , mkDisplayContext :: forall m. MonadIO m => Output -> DisplayRegion -> m DisplayContext
+    -- | Ring the terminal bell if supported.
+    , ringTerminalBell :: forall m. MonadIO m => m ()
+    -- | Returns whether the terminal has an audio bell feature.
+    , supportsBell :: forall m. MonadIO m => m Bool
     }
 
 displayContext :: MonadIO m => Output -> DisplayRegion -> m DisplayContext
@@ -151,13 +166,20 @@ outputPicture dc pic = liftIO $ do
         out = (if manipCursor then writeHideCursor dc else mempty)
               `mappend` writeOutputOps dc initialAttr diffs ops
               `mappend`
-                (case picCursor pic of
+                (let (w,h) = contextRegion dc
+                     clampX = max 0 . min (w-1)
+                     clampY = max 0 . min (h-1) in
+                 case picCursor pic of
                     _ | not manipCursor -> mempty
-                    NoCursor             -> mempty
+                    NoCursor            -> mempty
+                    AbsoluteCursor x y ->
+                        writeShowCursor dc `mappend`
+                        writeMoveCursor dc (clampX x) (clampY y)
                     Cursor x y           ->
                         let m = cursorOutputMap ops $ picCursor pic
                             (ox, oy) = charToOutputPos m (x,y)
-                        in writeShowCursor dc `mappend` writeMoveCursor dc ox oy
+                        in writeShowCursor dc `mappend`
+                           writeMoveCursor dc (clampX ox) (clampY oy)
                 )
     -- ... then serialize
     outputByteBuffer (contextDevice dc) (writeToByteString out)
